/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_DSC_H
#define SDXI_DSC_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))


/* From Table 63: DSC_GENERIC SDXI Descriptor Common Header and Footer For */
struct  SDXI_DSC_GENERIC {
    uint32_t  opcode;                 /* 000:000, vl                       */
                                      /* 001:001, se                       */
                                      /* 002:002, fe                       */
                                      /* 003:003, ch                       */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype                  */
                                      /* 026:016, type                     */
                                      /* 031:027, rsvd                     */
    uint8_t   operation[52];          /* 447:032,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 66: DSC_DMAB_NOP Descriptor Format */
struct  SDXI_DSC_DMAB_NOP {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = 0                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x01           */
                                      /* 026:016, type = 0x001             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0[52];             /* 447:032,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 67: DSC_DMAB_WRT_IMM Descriptor Format */
struct  SDXI_DSC_DMAB_WRT_IMM {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x02           */
                                      /* 026:016, type = 0x001             */
                                      /* 031:027, rsvd                     */
    uint32_t  size;                   /* 036:032, bsize                    */
                                      /* 063:037, rsvd                     */
    uint8_t   attr;                   /* 067:064, attr_dst                 */
                                      /* 071:068, rsvd                     */
    uint8_t   rsvd_0[3];              /* 095:072,                          */
    uint16_t  akey0;                  /* 111:096, akey0_dst                */
    uint8_t   rsvd_1[2];              /* 127:112,                          */
    uint64_t  addr0;                  /* 191:128, (dst)                    */
    uint8_t   data[32];               /* 447:192,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 68: DSC_DMAB_COPY Descriptor Format */
struct  SDXI_DSC_DMAB_COPY {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x03           */
                                      /* 026:016, type = 0x001             */
                                      /* 031:027, rsvd                     */
    uint32_t  size;                   /* 063:032,                          */
    uint8_t   attr;                   /* 067:064, attr_src                 */
                                      /* 071:068, attr_dst                 */
    uint8_t   rsvd_0[3];              /* 095:072,                          */
    uint16_t  akey0;                  /* 111:096, (src)                    */
    uint16_t  akey1;                  /* 127:112, (dst)                    */
    uint64_t  addr0;                  /* 191:128, (src)                    */
    uint64_t  addr1;                  /* 255:192, (dst)                    */
    uint8_t   rsvd_1[24];             /* 447:256,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 69: DSC_DMAB_REPCOPY Descriptor Format */
struct  SDXI_DSC_DMAB_REPCOPY {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x04           */
                                      /* 026:016, type = 0x001             */
                                      /* 031:027, rsvd                     */
    uint32_t  size;                   /* 043:032, rsvd                     */
                                      /* 052:044, nsize                    */
                                      /* 063:053, rsvd                     */
    uint8_t   attr;                   /* 067:064, attr_src                 */
                                      /* 071:068, attr_dst                 */
    uint8_t   rsvd_0[3];              /* 095:072,                          */
    uint16_t  akey0;                  /* 111:096, (src)                    */
    uint16_t  akey1;                  /* 127:112, (dst)                    */
    uint64_t  addr0;                  /* 128:128, az                       */
                                      /* 139:129, rsvd                     */
                                      /* 191:140, addr0_src                */
    uint64_t  addr1;                  /* 203:192, rsvd                     */
                                      /* 255:204, addr1_dst                */
    uint32_t  repeat;                 /* 267:256, rsvd                     */
                                      /* 287:268, num                      */
    uint8_t   rsvd_1[20];             /* 447:288,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 610: DSC_ATM Operation Descriptor Format */
struct  SDXI_DSC_ATM {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype =                */
                                      /*       SWAP: 0x01,     UADD: 0x02, */
                                      /*       USUB: 0x03,      AND: 0x05, */
                                      /*         OR: 0x06,      XOR: 0x07, */
                                      /*       SMIN: 0x08,     SMAX: 0x09, */
                                      /*       UMIN: 0x0A,     UMAX: 0x0B, */
                                      /*    UCLAMPI: 0x0C,  UCLAMPD: 0x0D, */
                                      /*    CMPSWAP: 0x0E,                 */
                                      /* 026:016, type = 0x003             */
                                      /* 031:027, rsvd                     */
    uint32_t  size;                   /* 033:032, rsvd                     */
                                      /* 036:034, osz                      */
                                      /* 063:037, rsvd                     */
    uint8_t   attr;                   /* 067:064, attr_dst                 */
                                      /* 071:068, rsvd                     */
    uint8_t   rsvd_0[3];              /* 095:072,                          */
    uint16_t  akey0;                  /* 111:096, (dst)                    */
    uint8_t   rsvd_1[2];              /* 127:112,                          */
    uint64_t  addr0;                  /* 129:128, rsvd                     */
                                      /* 130:130, n                        */
                                      /* 191:131, addr0_dst                */
    uint64_t  op1;                    /* 255:192,                          */
    uint64_t  op2;                    /* 319:256,                          */
    uint64_t  ret_data_ptr;           /* 320:320, nr                       */
                                      /* 321:321, rsvd                     */
                                      /* 383:322, ret_data_ptr             */
    uint8_t   rsvd_2[8];              /* 447:384,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 612: DSC_INTR Descriptor Format */
struct  SDXI_DSC_INTR {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x00           */
                                      /* 026:016, type = 0x004             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0[8];              /* 095:032,                          */
    uint16_t  akey;                   /* 111:096,                          */
    uint8_t   rsvd_1[42];             /* 447:112,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 613: DSC_VENDOR Descriptor Format */
struct  SDXI_DSC_VENDOR {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype                  */
                                      /* 026:016, type                     */
                                      /* 031:027, rsvd                     */
    uint8_t   vendor[52];             /* 447:032,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 614: DSC_CXT_START Descriptor Format */
struct  SDXI_DSC_CXT_START {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = 1                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype =                */
                                      /*         NM: 0x03,       RS: 0x08, */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0;                 /* 039:032,                          */
    uint8_t   vflags;                 /* 045:040, rsvd                     */
                                      /* 046:046, dv                       */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint16_t  cxt_start;              /* 079:064,                          */
    uint16_t  cxt_end;                /* 095:080,                          */
    uint8_t   rsvd_1[4];              /* 127:096,                          */
    uint64_t  db_value;               /* 191:128,                          */
    uint8_t   rsvd_2[32];             /* 447:192,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 615: DSC_CXT_STOP Descriptor Format */
struct  SDXI_DSC_CXT_STOP {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = 1                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x04           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0;                 /* 039:032,                          */
    uint8_t   vflags;                 /* 044:040, rsvd                     */
                                      /* 045:045, hs                       */
                                      /* 046:046, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint16_t  cxt_start;              /* 079:064,                          */
    uint16_t  cxt_end;                /* 095:080,                          */
    uint8_t   rsvd_1[44];             /* 447:096,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 616: DSC_ADM_AKEY_UPD Descriptor Format */
struct  SDXI_DSC_ADM_AKEY_UPD {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x02           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0;                 /* 039:032,                          */
    uint8_t   vflags;                 /* 046:040, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint16_t  cxt_start;              /* 079:064,                          */
    uint16_t  cxt_end;                /* 095:080,                          */
    uint16_t  akey_start;             /* 111:096,                          */
    uint16_t  akey_end;               /* 127:112,                          */
    uint8_t   rsvd_1[40];             /* 447:128,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 618: DSC_CXT_UPD Descriptor Format */
struct  SDXI_DSC_CXT_UPD {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x01           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   cflags;                 /* 034:032, tgt_pcd                  */
                                      /* 039:035, rsvd                     */
    uint8_t   vflags;                 /* 046:040, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint16_t  cxt_start;              /* 079:064,                          */
    uint16_t  cxt_end;                /* 095:080,                          */
    uint8_t   rsvd_0[44];             /* 447:096,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 619: DSC_FN_UPD Operation Descriptor Format */
struct  SDXI_DSC_FN_UPD {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x00           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0;                 /* 039:032, rsvd                     */
    uint8_t   vflags;                 /* 046:040, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint8_t   rsvd_1[48];             /* 447:064,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 620: DSC_RKEY_UPD Descriptor Format */
struct  SDXI_DSC_RKEY_UPD {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x07           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0;                 /* 039:032, rsvd                     */
    uint8_t   vflags;                 /* 046:040, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint8_t   rsvd_1[4];              /* 095:064, rsvd                     */
    uint16_t  rkey_start;             /* 111:096,                          */
    uint16_t  rkey_end;               /* 127:112,                          */
    uint8_t   rsvd_2[40];             /* 447:128,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 622: DSC_SYNC Descriptor Format */
struct  SDXI_DSC_SYNC {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x06           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   cflags;                 /* 034:032, flt                      */
                                      /* 039:035, rsvd                     */
    uint8_t   vflags;                 /* 046:040, rsvd                     */
                                      /* 047:047, vf                       */
    uint16_t  vf_num;                 /* 063:048,                          */
    uint16_t  cxt_start;              /* 079:064,                          */
    uint16_t  cxt_end;                /* 095:080,                          */
    uint16_t  key_start;              /* 111:096,                          */
    uint16_t  key_end;                /* 127:112,                          */
    uint8_t   rsvd_0[40];             /* 447:128,                          */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 623: DSC_ADM_INTR Descriptor Format */
struct  SDXI_DSC_ADM_INTR {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 001:001, se = *                   */
                                      /* 002:002, fe = *                   */
                                      /* 003:003, ch = 0                   */
                                      /* 004:004, csr                      */
                                      /* 007:005, rsvd                     */
                                      /* 015:008, subtype = 0x05           */
                                      /* 026:016, type = 0x002             */
                                      /* 031:027, rsvd                     */
    uint8_t   rsvd_0[8];              /* 095:032,                          */
    uint16_t  intr_num;               /* 107:096, intr_num                 */
                                      /* 111:108, rsvd                     */
    uint8_t   rsvd_1[42];             /* 447:112, "                        */
    uint64_t  csb_ptr;                /* 448:448, np                       */
                                      /* 452:449, rsvd                     */
                                      /* 511:453, csb_ptr                  */
} _SDXI_PACKED _SDXI_ALIGN(64);

#endif /* SDXI_DSC_H */
