/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_OPCODES_H
#define SDXI_OPCODES_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))

/* Opcode Macros */
#define SDXI_OPCODES_MASK        0x07FFFF0Fu
#define SDXI_OPCODES(a)          (uint32_t)((a) & OPCODES_MASK)
#define SDXI_OPCODES_VL          0x1u
#define SDXI_OPCODES_SE          0x2u
#define SDXI_OPCODES_FE          0x4u
#define SDXI_OPCODES_CH          0x8u

/* Opcodes */
#define SDXI_OP_DMAB_NOP           0x00010100u
#define SDXI_OP_DMAB_WRT_IMM       0x00010200u
#define SDXI_OP_DMAB_COPY          0x00010300u
#define SDXI_OP_DMAB_REPCOPY       0x00010400u
#define SDXI_OP_FN_UPD             0x00020000u
#define SDXI_OP_CXT_UPD            0x00020100u
#define SDXI_OP_ADM_AKEY_UPD       0x00020200u
#define SDXI_OP_CXT_START_NM       0x00020304u
#define SDXI_OP_CXT_STOP           0x00020404u
#define SDXI_OP_ADM_INTR           0x00020500u
#define SDXI_OP_SYNC               0x00020600u
#define SDXI_OP_RKEY_UPD           0x00020700u
#define SDXI_OP_CXT_START_RS       0x00020804u
#define SDXI_OP_ATM_SWAP           0x00030100u
#define SDXI_OP_ATM_UADD           0x00030200u
#define SDXI_OP_ATM_USUB           0x00030300u
#define SDXI_OP_ATM_AND            0x00030500u
#define SDXI_OP_ATM_OR             0x00030600u
#define SDXI_OP_ATM_XOR            0x00030700u
#define SDXI_OP_ATM_SMIN           0x00030800u
#define SDXI_OP_ATM_SMAX           0x00030900u
#define SDXI_OP_ATM_UMIN           0x00030a00u
#define SDXI_OP_ATM_UMAX           0x00030b00u
#define SDXI_OP_ATM_UCLAMPI        0x00030c00u
#define SDXI_OP_ATM_UCLAMPD        0x00030d00u
#define SDXI_OP_ATM_CMPSWAP        0x00030e00u
#define SDXI_OP_INTR               0x00040000u

#endif /* SDXI_OPCODES_H */
